﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment {
  [Item("N-Move Maker", "Performs an N-Move.")]
  [StorableClass]
  public class NMoveMaker : SingleSuccessorOperator, IQualityAwareGQAPOperator, IMoveQualityAwareGQAPOperator, IGQAPNMoveOperator, IMoveMaker {

    public ILookupParameter<IntegerVector> AssignmentParameter {
      get { return (ILookupParameter<IntegerVector>)Parameters["Assignment"]; }
    }
    public ILookupParameter<NMove> MoveParameter {
      get { return (ILookupParameter<NMove>)Parameters["Move"]; }
    }
    public ILookupParameter<DoubleValue> QualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["Quality"]; }
    }
    public ILookupParameter<Evaluation> EvaluationParameter {
      get { return (ILookupParameter<Evaluation>)Parameters["Evaluation"]; }
    }
    public ILookupParameter<DoubleValue> MoveQualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["MoveQuality"]; }
    }
    public ILookupParameter<Evaluation> MoveEvaluationParameter {
      get { return (ILookupParameter<Evaluation>)Parameters["MoveEvaluation"]; }
    }

    [StorableConstructor]
    protected NMoveMaker(bool deserializing) : base(deserializing) { }
    protected NMoveMaker(NMoveMaker original, Cloner cloner) : base(original, cloner) { }
    public NMoveMaker()
      : base() {
      Parameters.Add(new LookupParameter<IntegerVector>("Assignment", GQAPSolutionCreator.AssignmentDescription));
      Parameters.Add(new LookupParameter<NMove>("Move", GQAPNMoveGenerator.MoveDescription));
      Parameters.Add(new LookupParameter<DoubleValue>("Quality", ""));
      Parameters.Add(new LookupParameter<Evaluation>("Evaluation", GQAP.EvaluationDescription));
      Parameters.Add(new LookupParameter<DoubleValue>("MoveQuality", GQAPNMoveEvaluator.MoveQualityDescription));
      Parameters.Add(new LookupParameter<Evaluation>("MoveEvaluation", GQAPNMoveEvaluator.MoveEvaluationDescription));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new NMoveMaker(this, cloner);
    }

    public static void Apply(IntegerVector vector, NMove move) {
      for (var i = 0; i < move.N; i++) {
        var equip = move.Indices[i];
        vector[equip] = move.Reassignments[equip] - 1; // location is given 1-based in reassignments
      }
    }

    public override IOperation Apply() {
      Apply(AssignmentParameter.ActualValue, MoveParameter.ActualValue);
      QualityParameter.ActualValue.Value = MoveQualityParameter.ActualValue.Value;
      EvaluationParameter.ActualValue = MoveEvaluationParameter.ActualValue;
      return base.Apply();
    }
  }
}
